/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "burndevicewidget.h"
#include "k3bemptydiscwaiter.h"
#include "burnsidebarplugin.h"
#include "burninfomessagedialog.h"
#include <k3bjob.h>
#include <k3bdevice.h>
#include <k3bdoc.h>
#include <kstdguiitem.h>
#include <qlabel.h>
#include <qlayout.h>
#include <kprogress.h>
#include <qslider.h>
#include <qwidgetstack.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <qstring.h>
#include <qmutex.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <qtimer.h>
#include <klocale.h>
#include <qvbox.h>
#include <kpushbutton.h>
#include <qstringlist.h>

BurnDeviceWidget::BurnDeviceWidget(K3bDevice::Device *device, const QWidget *dragSource, BurnSidebarPlugin *plugin, QWidget *parent, const char *name)
 : QFrame(parent, name),
   m_device( device ),
   m_dragSource( dragSource ),
   m_plugin( plugin ),
   m_burnJob(0)
{

  setFrameStyle(  QFrame::StyledPanel );
  setLineWidth( 2 );
  setMargin( 3 );

  setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

  setAcceptDrops( true );
  //initialise all the GUI stuff
  QPixmap pixmap = KGlobal::iconLoader()->loadIcon( "cdwriter_unmount", KIcon::Desktop, KIcon::SizeLarge );
  m_pixmapLabel = new QLabel( this );
  m_pixmapLabel->setPixmap( pixmap );
  m_pixmapLabel->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Expanding );

  m_widgetStack = new QWidgetStack( this );
  
  m_hLayout = new QHBoxLayout( this );

  m_hLayout->addWidget( m_pixmapLabel );
  m_hLayout->addWidget( m_widgetStack );
  m_hLayout->setSpacing( 2 );
  m_hLayout->setMargin( 4 );

  m_mainVBox = new QVBox( this );
  m_mainVBox->setSpacing(3);

  m_textLabel = new QLabel( m_mainVBox );
  m_textLabel->setTextFormat( Qt::RichText );
  m_textLabel->setText( "Drag here to burn to " + device->vendor() + " " + device->description());
  m_textLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum );
  m_textLabel->setMargin(0);

  int speed = BurnSidebarPlugin::config()->readNumEntry( device->description() + "-speed", device->maxWriteSpeed() );

  m_mainHBox = new QHBox( m_mainVBox );
  m_mainHBox->setSpacing(3);

  m_speedLabel = new QLabel( m_mainHBox );
  m_speedLabel->setText( i18n("Speed: %1").arg( (int)(speed/175) ) );
  m_speedSlider = new QSlider( 175, device->maxWriteSpeed(), 1750, speed, Qt::Horizontal,  m_mainHBox );
  m_speedSlider->setLineStep( 175 );
  
  connect( m_speedSlider, SIGNAL( valueChanged( int ) ),
           this, SLOT( speedSliderChanged( int ) ) );

  m_progressBox = new QWidget( this );
  QVBoxLayout *progressLayout = new QVBoxLayout( m_progressBox );
  m_progress = new KProgress( m_progressBox );
  m_progress->setFormat( "Burning... %p%" );
  m_progress->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
  progressLayout->addStretch(1);
  progressLayout->add( m_progress );
  progressLayout->addStretch(1);

  m_errorBox = new QWidget( this );
  m_errorBox->setBackgroundColor( QColor( 0xff, 0 ,0 ) );
  m_errorLabel = new QLabel( m_errorBox );
  m_errorLabel->setBackgroundColor( QColor( 0xff, 0, 0 ) );
  m_errorLabel->setTextFormat( Qt::RichText );
  m_errorLabel->setMargin(0);
  m_errorLabel->setPaletteForegroundColor( QColor( 0xff, 0xff, 0xff ) );
  m_errorLabel->setText( i18n( "Burning failed" ) );
  QFont errorFont = m_errorLabel->font();
  errorFont.setBold( true );
  m_errorLabel->setFont( errorFont );


  m_errorOkButton = new KPushButton( KStdGuiItem::ok(), m_errorBox );
  m_errorOkButton->setBackgroundColor( QColor( 0xff, 0 ,0 ) );
  connect( m_errorOkButton, SIGNAL( clicked() ),
           this, SLOT( slotErrorTimeout() ) );
  m_errorDetailsButton = new QPushButton( m_errorBox );
  m_errorDetailsButton->setText( i18n( "Details..." ) );
  m_errorDetailsButton->setBackgroundColor( QColor( 0xff, 0 ,0 ) );
  connect( m_errorDetailsButton, SIGNAL( clicked() ),
           this, SLOT( displayInfoMessages() ) );

  QVBoxLayout *errorLayout = new QVBoxLayout( m_errorBox );
  errorLayout->addStretch(1);
  errorLayout->add( m_errorLabel );
  errorLayout->addStretch(1);
  QHBoxLayout *errorButtonLayout = new QHBoxLayout( errorLayout );
  errorButtonLayout->setSpacing( 10 );
  errorButtonLayout->add( m_errorOkButton );
  errorButtonLayout->add( m_errorDetailsButton );
  errorLayout->addStretch(1);

  m_burnVBox = new QWidget( this );
  QVBoxLayout *burnVLayout = new QVBoxLayout( m_burnVBox );
  burnVLayout->addStretch( 1 );
  m_burnLabel = new QLabel( i18n("Ready to begin burning" ), m_burnVBox );
  burnVLayout->add( m_burnLabel );
  burnVLayout->addStretch( 1 );
  m_burnHBox = new QHBox( m_burnVBox );
  burnVLayout->add( m_burnHBox );
  m_burnHBox->setSpacing(10);
  m_burnButton = new QPushButton( i18n( "Burn" ), m_burnHBox );
  m_burnButton->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
  m_burnCancelButton = new QPushButton( i18n( "Cancel" ), m_burnHBox );
  m_burnCancelButton->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
  burnVLayout->addStretch( 1 );

  connect( m_burnButton, SIGNAL( clicked() ),
           this, SLOT( startBurning() ) );
  connect( m_burnCancelButton, SIGNAL( clicked() ),
           this, SLOT( cancelClicked() ) );

  m_widgetStack->addWidget( m_progressBox );
  m_widgetStack->addWidget( m_mainVBox );
  m_widgetStack->addWidget( m_errorBox );
  m_widgetStack->addWidget( m_burnVBox );
  m_widgetStack->raiseWidget( m_mainVBox );

  m_infoMessages = new QStringList();
}

int BurnDeviceWidget::waitForMedia( K3bDevice::Device* dev,
                                    int mediaState,
                                    int mediaType,
                                    const QString& message)
{
  return K3bEmptyDiscWaiter::wait( dev, mediaState, mediaType, message, this );
}
void BurnDeviceWidget::displayInfoMessages()
{
  BurnInfoMessageDialog( *m_infoMessages, this ).exec();
}
bool BurnDeviceWidget::questionYesNo( const QString& text,
					  const QString& caption )
{
  return ( KMessageBox::questionYesNo( this, text, caption ) == KMessageBox::Yes );
}
BurnDeviceWidget::~BurnDeviceWidget()
{
  BurnSidebarPlugin::config()->sync();
  delete m_infoMessages;
}

void BurnDeviceWidget::speedSliderChanged( int newValue )
{
  m_speedLabel->setText( i18n( "Speed: %1" ).arg( (int)(newValue/175) ) );
  BurnSidebarPlugin::config()->writeEntry( m_device->description() + "-speed", newValue );
}
void BurnDeviceWidget::dragEnterEvent( QDragEnterEvent *e )
{
  e->accept();
}

void BurnDeviceWidget::dropEvent( QDropEvent *de )
{
  if( de->source() == m_dragSource )
  {
    m_widgetStack->raiseWidget( m_burnVBox );

    setAcceptDrops( false );

  }
}

void BurnDeviceWidget::cancelClicked()
{
  m_widgetStack->raiseWidget( m_mainVBox );			
  setAcceptDrops(true);
}

void BurnDeviceWidget::startBurning()
{
    K3bDoc *document = m_plugin->k3bDoc();
    if( !document )
    {
      return;
    }

    document->setBurner( m_device );
    document->setSpeed( m_speedSlider->value() );
//#ifndef BURN_FINAL
    //document->setDummy( true );
//#endif
    Q_ASSERT( m_burnJob == 0 ); //we don't allow creating a new burnjob 
                               //if one already exists
    m_burnJob = document->newBurnJob( this, this );

    connect( m_burnJob, SIGNAL( started() ), this, SLOT( slotStarted() ) );
    connect( m_burnJob, SIGNAL( percent(int) ), m_progress, SLOT( setValue(int ) ) );
    connect( m_burnJob, SIGNAL( finished(bool) ), this, SLOT( slotFinished(bool) ) );
    connect( m_burnJob, SIGNAL( infoMessage( const QString&, int) ), this, SLOT( slotInfoMessage( const QString&, int ) ) );

    m_burnJob->start();
}


void BurnDeviceWidget::slotInfoMessage(const QString &message, int type )
{
  kdDebug() << "Info message: " << message << endl;
  
  m_infoMessages->append( message );

  if( type == K3bJob::ERROR )
  {
    m_widgetStack->raiseWidget( m_errorBox );
    setBackgroundColor( QColor( 0xff, 0, 0 ) );
    m_pixmapLabel->setBackgroundColor( QColor( 0xff, 0, 0 ) );
  }
}

void BurnDeviceWidget::slotErrorTimeout()
{
  m_widgetStack->raiseWidget( m_mainVBox );
  setAcceptDrops( true );
  setBackgroundColor( m_textLabel->backgroundColor() );
  m_pixmapLabel->setBackgroundColor( m_textLabel->backgroundColor() );
}

void BurnDeviceWidget::slotStarted()
{
  kdDebug() << "void BurnDeviceWidget::slotStarted()" << endl;
  m_progress->setValue(0);
  m_widgetStack->raiseWidget( m_progressBox);
  m_infoMessages->clear();
}

void BurnDeviceWidget::slotFinished( bool success )
{
  kdDebug() << "void BurnDeviceWidget::slotFinished( bool success = " << success << " )" << endl;

  if( m_widgetStack->visibleWidget() == m_progressBox )
  {
    m_widgetStack->raiseWidget( m_mainVBox );
    setAcceptDrops( true );
  }

  delete m_burnJob->doc();
  delete m_burnJob;
  m_burnJob = 0;
}
#include "burndevicewidget.moc"
